package ink.wgink.login.oauth2.server.converter;

import ink.wgink.interfaces.consts.ISystemConstant;
import ink.wgink.interfaces.role.IRoleBaseService;
import ink.wgink.pojo.bos.LoginUser;
import ink.wgink.pojo.bos.UserInfoBO;
import ink.wgink.pojo.dtos.user.UserAttrInfoDTO;
import ink.wgink.service.user.service.IUserService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.GrantedAuthority;
import org.springframework.security.oauth2.provider.token.UserAuthenticationConverter;

import java.util.*;

/**
 * @ClassName: UserAuthConverter
 * @Description: 重写用户认证
 * @Author: WangGeng
 * @Date: 2019/2/27 4:57 PM
 * @Version: 1.0
 **/
public class UserAuthConverter implements UserAuthenticationConverter {

    private static final Logger LOG = LoggerFactory.getLogger(UserAuthConverter.class);
    private IUserService userService;

    public UserAuthConverter() {
    }

    @Override
    public Map<String, ?> convertUserAuthentication(Authentication authentication) {
        Map<String, Object> response = new LinkedHashMap<>();
        response.put("user_name", authentication.getName());
        // 删除token中的权限信息，通过客户端请求获取，减少accessToken长度
        LoginUser loginUser = (LoginUser) authentication.getPrincipal();
        // 传递登录用户
        Map<String, Object> userInfo = new HashMap<>(4);
        userInfo.put("userId", loginUser.getUserId());
        userInfo.put("username", loginUser.getUsername());
        userInfo.put("userName", loginUser.getUserName());
        userInfo.put("userPhone", loginUser.getUserPhone());
        userInfo.put("userAvatar", loginUser.getUserAvatar());
        userInfo.put("userEmail", loginUser.getUserEmail());
        userInfo.put("roles", loginUser.getRoles());
        userInfo.put("departments", loginUser.getDepartments());
        userInfo.put("groups", loginUser.getGroups());
        userInfo.put("getPositions", loginUser.getPositions());
        userInfo.put("expandData", loginUser.getExpandData());
        response.put("user_info", userInfo);
        return response;
    }

    @Override
    public Authentication extractAuthentication(Map<String, ?> map) {
        // 解析客户端的权限请求
        Object principal = map.get("user_name");
        if (!Objects.isNull(principal)) {
            Collection<GrantedAuthority> authorities;
            String userName = principal.toString();
            // 包含用户信息，则直接抽取其中的用户信息
            Map<?, ?> userInfo = (Map<String, Object>) map.get("user_info");

            UserInfoBO userInfoBO = new UserInfoBO();
            userInfoBO.setUserId(userInfo.get("userId").toString());
            userInfoBO.setUserUsername(userInfo.get("username").toString());
            userInfoBO.setUserName(userInfo.get("userName").toString());
            userInfoBO.setUserPhone(userInfo.get("userPhone") == null ? "" : userInfo.get("userPhone").toString());
            userInfoBO.setUserAvatar(userInfo.get("userAvatar") == null ? "" : userInfo.get("userAvatar").toString());
            userInfoBO.setUserEmail(userInfo.get("userEmail") == null ? "" : userInfo.get("userEmail").toString());
            userInfoBO.
            if (ISystemConstant.ADMIN.equals(userName)) {
            } else {
                UserAttrInfoDTO userAttrInfoDTO = userService.getUserAttrInfoByUserId(userInfoBO.getUserId());
                userInfoBO.setDepartments(userAttrInfoDTO.getDepartments());
                userInfoBO.setRoles(userAttrInfoDTO.getRoles());
                userInfoBO.setGroups(userAttrInfoDTO.getGroups());
                userInfoBO.setPositions(userAttrInfoDTO.getPositions());
                userInfoBO.setDataAuthority(userAttrInfoDTO.getDataAuthority());
                userInfoBO.setDataAuthorityUserIds(userAttrInfoDTO.getDataAuthorityUserIds());
                userInfoBO.setBaseDepartmentIds(userAttrInfoDTO.getBaseDepartmentIds());


                // 设置权限
                authorities = getAuthorities(userAttrInfoDTO.getRoles());
            }
            principal = userInfoBO;
            LOG.debug("获取用户权限");
            return new UsernamePasswordAuthenticationToken(principal, "N/A", authorities);
        } else {
            return null;
        }
    }

    public IUserService getUserService() {
        return userService;
    }

    public void setUserService(IUserService userService) {
        this.userService = userService;
    }
}
